import React, { Component } from 'react';
import { Marker, /*Popup, */Tooltip, withLeaflet } from "react-leaflet";
import { Icon, LatLng } from 'leaflet';
import bearing from '@turf/bearing';
import PubSub from 'pubsub-js';
import 'leaflet.marker.slideto/Leaflet.Marker.SlideTo.js'

class MoveableMarker extends Component{
    componentDidMount(){       
        this.card_id =  this.props.data.card_id;
		
		PubSub.subscribe('map.marker.move', (msg, data) => {
            if(this.card_id === data.id){
                let oldLatlng = this.marker.leafletElement.getLatLng();
                let newLatlng = new LatLng(data.lat, data.lng);

                let b = bearing([oldLatlng.lat, oldLatlng.lng], [newLatlng.lat, newLatlng.lng]) + 90;
                this.marker.leafletElement.setRotationAngle(b * 0.33333);
                
                this.marker.leafletElement.slideTo([data.lat, data.lng], {
                    duration: data.duration,
                    keepAtCenter: false
                });
            }
        });
    }

    render(){
        return <>
            <Marker 
                position={[this.props.data.lat, this.props.data.lng] } 
                icon={new Icon({
                    iconUrl: window.EDITOR_URL + this.props.data.src,
                    iconSize: [20,20],
                    iconAnchor: [10,10]
                })}
                name={`${this.card_id}`} key={this.card_id}
                ref={marker => this.marker = marker}
            >
                {this.props.data.info &&
                    <Tooltip>
                        <div dangerouslySetInnerHTML={{ __html: this.props.data.info }} />
                    </Tooltip>
                }
            </Marker>;
        </>
    }
}

export default withLeaflet(MoveableMarker);